#!/usr/bin/python

"""
QueryTool

Description:
Python script to query data from a KexiDB datasource and view it.

Author:
Sebastian Sauer <mail@dipe.org>

Copyright:
Dual-licensed under LGPL v2+higher and the BSD license.
"""

import qt
import sys
import os

class Configuration:
	file = None #"/home/snoopy/test.kexi"
	query = "" #"select * from table1;"

class Page(qt.QVBox):
	def __init__(self, dialog):
		self.dialog = dialog
		self.visiblecount = 0
		global qt
		qt.QVBox.__init__(self, self.dialog.tab)
		self.setMargin(6)
		self.setSpacing(6)

		splitter = qt.QSplitter(qt.Qt.Vertical, self)

		editbox = qt.QVBox(splitter)
		self.edit = qt.QTextEdit(editbox)
		global Configuration
		qt.QObject.connect(self.edit, qt.SIGNAL("textChanged()"), self.editChanged)
		self.edit.setFocus()
		self.parserlabel = qt.QLabel("-", editbox)
		splitter.setCollapsible(editbox,False)
		splitter.setResizeMode(editbox, qt.QSplitter.KeepSize)
		self.edit.setText(Configuration.query)

		self.listview = qt.QListView(splitter)
		self.listview.setAllColumnsShowFocus(True)
		#splitter.setResizeMode(self.listview, qt.QSplitter.Stretch)
		#self.listview.setMinimumSize( self.listview.sizeHint() )

		filterbox = qt.QHBox(self)
		qt.QLabel("Filter:", filterbox)
		self.filteredit = qt.QLineEdit(filterbox)
		self.filtertimer = qt.QTimer(self)
		qt.QObject.connect(self.filtertimer, qt.SIGNAL("timeout()"), self.filterTimeout)
		qt.QObject.connect(self.filteredit, qt.SIGNAL("textChanged(const QString&)"), self.filterChanged)

		statusbox = qt.QHBox(self)
		statusbox.setFrameStyle(qt.QFrame.Sunken|qt.QFrame.Panel)
		self.connectlabel = qt.QLabel(statusbox)
		self.totallabel = qt.QLabel(statusbox)
		self.totallabel.setAlignment(qt.Qt.AlignRight)
		self.updateStates()

	def updateStates(self):
		if not hasattr(self,'connectiondata'): self.connectiondata = self.dialog.kexidbconnection.data()
		self.connectlabel.setText("Connected with %s" % self.connectiondata.serverInfoString())
		self.totallabel.setText("Records: %i of %i" % (self.visiblecount,self.listview.childCount()))

	def editChanged(self):
		try:
			text = str(self.edit.text()).strip()
			if text != "":
				self.dialog.kexidbparser.parse(text)
				op = self.dialog.kexidbparser.operation()
				if op == 'Error':
					self.parserlabel.setText(self.dialog.kexidbparser.errorMsg())
				elif op != 'Select':
					self.parserlabel.setText("Invalid operation: %s" % op)
				else:
					self.parserlabel.setText("Valid SQL")
				return
		except:
			import sys, traceback
			print "".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) )
		self.parserlabel.setText("")

	def filterChanged(self, text):
		self.filtertimer.stop()
		self.filtertimer.start(500, True)

	def itemIsVisible(self, item):
		filtertext = str( self.filteredit.text() )
		visible = (filtertext == "")
		if not visible:
			filters = [ f for f in filtertext.split(' ') if len(f) > 0 ]
			for i in range( self.listview.columns() ):
				for f in filters:
					if item.text(i).find(f) >= 0:
						visible = True
						break
		return visible

	def filterTimeout(self):
		self.visiblecount = 0
		filtertext = str(self.filteredit.text())
		item = self.listview.firstChild()
		while item:
			if self.itemIsVisible(item):
				self.visiblecount += 1
				item.setVisible(True)
			else:
				item.setVisible(False)
			item = item.nextSibling()
		self.updateStates()

	def saveClicked(self):
		import csv
		filename = str(qt.QFileDialog.getSaveFileName("data.csv","*.csv *.txt;;*",self.dialog))
		if filename == "": return
		qt.QApplication.setOverrideCursor( qt.QCursor(qt.Qt.WaitCursor) )
		try:
			file = open(filename, "w")
			csvwriter = csv.writer(file, dialect="excel")
			columncount = self.listview.columns()
			item = self.listview.firstChild()
			while item:
				if item.isVisible():
					record = []
					for i in range(columncount):
						record.append( unicode(item.text(i)).encode("utf-8") )
					csvwriter.writerow(record)
				item = item.nextSibling()
		except:
			import sys, traceback
			exception = "<br>".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) )
			qt.QMessageBox.critical(self,"Save failed.", "<qt>%s</qt>" % exception)
			pass
		qt.QApplication.restoreOverrideCursor()

	def executeClicked(self):
		self.totallabel.setText("Execution is running...")
		self.totallabel.update()
		self.listview.clear()
		while self.listview.columns() > 0: self.listview.removeColumn(0)
		self.visiblecount = 0
		qt.QApplication.setOverrideCursor( qt.QCursor(qt.Qt.WaitCursor) )

		class QueryException(Exception):
			def __init__(self, text): self.text = text

		try:
			query = str(self.edit.text())
			cursor = self.dialog.kexidbconnection.executeQueryString(query)
			if not cursor:
				raise QueryException("Failed to execute the query \"%s\".\n\n%s" % (query,self.dialog.kexidbconnection.lastError()))

			for i in range( cursor.fieldCount() ):
				self.listview.addColumn("#%i" % i)

			ok = cursor.moveFirst()
			while ok:
				item = qt.QListViewItem(self.listview)
				for i in range( cursor.fieldCount() ):
					v = cursor.value(i)
					if v != None: item.setText(i, str(v))
				if self.itemIsVisible(item):
					self.visiblecount += 1
				else:
					item.setVisible(False)
				ok = cursor.moveNext()

		except QueryException, (e):
			qt.QMessageBox.critical(self,"Query failed.", "<qt>%s</qt>" % e.text.replace("\n",'<br>'))
			pass
		except:
			import sys, traceback
			exception = "<br>".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) )
			qt.QMessageBox.critical(self,"Query failed.", "<qt>%s</qt>" % exception)
			pass
		qt.QApplication.restoreOverrideCursor()
		self.updateStates()

class Dialog(qt.QDialog):
	def __init__(self, kexidbconnection):
		import qt
		qt.QDialog.__init__(self, qt.qApp.mainWidget(), "Dialog", 1, qt.Qt.WDestructiveClose)
		self.kexidbconnection = kexidbconnection
		self.kexidbparser = self.kexidbconnection.parser()
		self.setCaption("Query Tool")
		layout = qt.QVBoxLayout(self)
		box = qt.QVBox(self)
		box.setMargin(6)
		box.setSpacing(6)
		layout.addWidget(box)

		btnbox = qt.QHBox(box)
		btnbox.setSpacing(6)
		self.newbtn = qt.QPushButton("New", btnbox)
		qt.QObject.connect(self.newbtn, qt.SIGNAL("clicked()"), self.newClicked)
		self.copybtn = qt.QPushButton("Copy", btnbox)
		qt.QObject.connect(self.copybtn, qt.SIGNAL("clicked()"), self.copyClicked)
		self.rembtn = qt.QPushButton("Remove", btnbox)
		qt.QObject.connect(self.rembtn, qt.SIGNAL("clicked()"), self.removeClicked)
		self.savebtn = qt.QPushButton("Save...", btnbox)
		qt.QObject.connect(self.savebtn, qt.SIGNAL("clicked()"), self.saveClicked)
		self.execbtn = qt.QPushButton("Execute", btnbox)
		self.execbtn.setDefault(True)
		qt.QObject.connect(self.execbtn, qt.SIGNAL("clicked()"), self.executeClicked)

		self.tab = qt.QTabWidget(box)
		#self.tab.setMargin(6)
		box.setStretchFactor(self.tab,1)

		box.setMinimumSize( qt.QSize(600,480) )
		self.pagecounter = 1
		self.addQueryPage()

	def addQueryPage(self):
		global Page
		page = Page(self)
		self.tab.addTab(page,"Query%i" % self.pagecounter)
		self.pagecounter += 1
		return page

	def updateStates(self):
		ok = self.tab.count() > 0
		self.copybtn.setEnabled(ok)
		self.rembtn.setEnabled(ok)
		self.savebtn.setEnabled(ok)
		self.execbtn.setEnabled(ok)

	def newClicked(self):
		page = self.addQueryPage()
		self.tab.setCurrentPage( self.tab.indexOf(page) )
		self.updateStates()

	def copyClicked(self):
		oldpage = self.tab.currentPage()
		newpage = self.addQueryPage()
		newpage.edit.setText( oldpage.edit.text() )
		self.tab.setCurrentPage( self.tab.indexOf(newpage) )
		self.updateStates()

	def removeClicked(self):
		page = self.tab.currentPage()
		if page == None: return
		self.tab.removePage(page)
		page.hide()
		page.destroy()
		self.updateStates()

	def saveClicked(self):
		self.tab.currentPage().saveClicked()

	def executeClicked(self):
		self.tab.currentPage().executeClicked()

def main():
	try:
		import krosskexidb
		kexidbdrivermanager = krosskexidb.DriverManager()
	except:
		raise "Failed to import the Kross KexiDB module. Please use krossrunner or Kexi to execute this python script."

	kexidbconnection = None
	try:
		import krosskexiapp
		keximainwindow = krosskexiapp.get("KexiAppMainWindow")
		kexidbconnection = keximainwindow.getConnection()
		connectiondata = kexidbconnection.data()
	except:
		global qt
		global Configuration

		file = Configuration.file
		if file == None:
			file = str( qt.QFileDialog.getOpenFileName("","*.kexi *.kexis *.kexic;;*") )
			if not file: sys.exit()

		connectiondata = kexidbdrivermanager.createConnectionDataByFile(file)
		if connectiondata == None:
			raise "Unsupported file."

		driver = kexidbdrivermanager.driver( connectiondata.driverName() )
		if driver == None:
			raise "Unsupported driver."

		kexidbconnection = driver.createConnection(connectiondata)
		print "Connected %s" % connectiondata.serverInfoString()

	if not kexidbconnection.isConnected():
		if not kexidbconnection.connect():
			raise "Failed to connect with \"%s\"" % connectiondata.serverInfoString()
	if not kexidbconnection.isDatabaseUsed():
		if not kexidbconnection.useDatabase( connectiondata.databaseName() ):
			if not kexidbconnection.useDatabase( connectiondata.fileName() ):
				raise "Failed to use database at \"%s\"." % connectiondata.serverInfoString()

	global Dialog
	dialog = Dialog(kexidbconnection)
	dialog.exec_loop()

main()
